<?php
declare(strict_types=1);
 
namespace Edara\Includes;

class ProductTaxRate extends ProductService {
 
    const SYNC_PRODUCT_TAX_RATE = 'sync_products_tax_rate';
    const EXTERNAL_PRODUCT_TAX_RATE = 'external_product_tax_rate';

    private $limit = 100;
    private $offset = 0;
    

    /**
     * Start sync products tax rate from edara ERP
     */
    public function startSyncProductsTaxRate(): void {

        if (true == get_transient(static::SYNC_PRODUCT_TAX_RATE)) {
            $this->syncProductsTaxRate();
            return;
        }

        $response = $this->api()->listProducts($this->limit, $this->offset);
        $data = [
            'limit' => $this->limit,
            'offset' => $this->offset,
            'total_count' => array_get($response, 'total_count')
        ];
        set_transient(static::SYNC_PRODUCT_TAX_RATE, $data);
        $this->syncProductsTaxRate($response);
    }

    /**
     * Syncing products tax rate from edara ERP
     * @param null $response
     */
    private function syncProductsTaxRate($response = null) {

        $transientData = get_transient(static::SYNC_PRODUCT_TAX_RATE);
        $this->offset = intval(array_get($transientData, 'offset')) ?: $this->offset;

        if (!$response) {
            $response = $this->api()->listProducts($this->limit, $this->offset);
        }

        if ((array_get($response, 'total_count') - $this->offset) == 0 || array_get($response, 'total_count') == null) {
            delete_transient(static::SYNC_PRODUCT_TAX_RATE);
            echo json_encode(['status' => 'done']);
            wp_die();
        }


        $data = [];
        $products = array_get($response, 'result');
        foreach ($products as $product) {
            $data[$product['id']] = $product['tax_rate'];
        }

        $external_ids = implode(', ', array_keys($data));

        global $wpdb;
        $internalProducts = $wpdb->get_results(
            "SELECT * FROM $wpdb->postmeta where meta_key = 'external_id' and meta_value in ($external_ids)"
        );

        foreach ($internalProducts as $internal_product) {
            update_post_meta($internal_product->post_id, static::EXTERNAL_PRODUCT_TAX_RATE, $data[$internal_product->meta_value]);
        }

        $totalCount = array_get($response, 'total_count');

        $data = [
            'limit' => $this->limit,
            'offset' => $this->offset + $this->limit,
            'total_count' => $totalCount,
        ];
        set_transient(static::SYNC_PRODUCT_TAX_RATE, $data);

        $data['offset'] = ($this->offset + $this->limit) > $totalCount ? $totalCount : ($this->offset + $this->limit);
        echo json_encode($data);

        wp_die();
    }

    /**
     * @return void
     */
    public function enqueueScripts(): void {
        if (true == get_transient(static::SYNC_PRODUCT_TAX_RATE)) {
            wp_enqueue_script(
                'syncProductsTaxRate',
                '/wp-content/plugins/connect-to-edara/assets/js/admin.js'
            );
        }
    }
}
